<?php

/**
 * 日历-查看任务进度，可以方便地修改任务开始时间和结束时间，更好地进行工作排期
 *
 * @author   riverdance@qq.com
 * @package  calendar
 * @version  $Id$
 */
class calendar extends control
{
    /**
     * Construct function, Set projects.
     *
     * @access public
     * @return void
     */
    public function __construct($moduleName = '', $methodName = '')
    {
        parent::__construct($moduleName, $methodName);
    }

    /**
     * The index page.
     *
     * @link http://zentaopms.coobar.org/calendar-index-1.html
     * @param  int $projectID
     * @access public
     * @return void
     */
    public function index($projectID = 0)
    {
        $projs = $this->_getProjectKVlist();
        $projsSelect = html::select('project_id', $projs, $projectID, "class='form-control' onchange='changeProject(this.value)'");

        $this->view->tasks = $this->_getTaskList($projectID);
        $this->view->projsSelect = $projsSelect;
        $this->display();
    }

    private function _getProjectKVlist()
    {
        $projs = [];
        $list = $this->loadModel('calendar')->getProjectList();
        if(!empty($list)){
            foreach($list as $data){
                $key = $data->id;
                $val = $data->name;
                $projs[$key] = $val;
            }
        }
        return $projs;
    }

    private function _getTaskList($projectID)
    {
        $tasks = [];
        $list = $this->loadModel('calendar')->getTaskList($projectID);
        if(!empty($list)){
            foreach($list as $data){
                $t = [];
                $t['id'] = $data->id;
                $t['title'] = '['.$data->assignedTo.']'.$data->name;
                if('' != $data->estStarted){
                    $t['start'] = $data->estStarted;
                }
                if('' != $data->deadline){
                    $t['end'] = $data->deadline;
                }
                $t['color'] = self::_getPersonColor($data->assignedTo);
                $tasks[] = $t;
            }
        }
        return $tasks;
    }

    private static function _getPersonColor($assignedTo)
    {
        $str = substr(md5($assignedTo), 0, 1);
        $list = [
        '0' => '#009900',
        '1' => '#00CC00',
        '2' => '#00FF00',
        '3' => '#330000',
        '4' => '#333300',
        '5' => '#336600',
        '6' => '#660000',
        '7' => '#660033',
        '8' => '#663300',
        '9' => '#660099',
        'a' => '#3399CC',
        'b' => '#3366CC',
        'c' => '#999933',
        'd' => '#99CC99',
        'e' => '#9999CC',
        'f' => '#999999',
        'A' => '#99FFCC',
        'B' => '#CC00FF',
        'C' => '#CC6633',
        'D' => '#CC0033',
        'E' => '#CCCC99',
        'F' => '#CCFF33',
        ];
        $color = isset($list[$str]) ? $list[$str] : '#000000';
        return $color;
    }

    /**
     * resize task
     *
     * @link http://zentaopms.coobar.org/calendar-resize-1.html
     * @param  int $taskID
     * @access public
     * @return void
     */
    public function resize($taskID=0)
    {
        $dayDelta = intval($_POST['dayDelta']);

        if(0 == $dayDelta){
            self::_error('没有任何修改');
        }

        $task = $this->loadModel('calendar')->getTask($taskID);
        if(empty($task)){
            self::_error('任务不存在');
        }

        $deadline = self::_calDay($task->deadline,$dayDelta);

        $result = $this->loadModel('calendar')->resizeTask($taskID,$deadline);
        if(!$result){
            self::_error('操作失败');
        }
        self::_success();
    }

    /**
     * move task
     *
     * @link http://zentaopms.coobar.org/calendar-move-1.html
     * @param  int $taskID
     * @access public
     * @return void
     */
    public function move($taskID=0)
    {
        $dayDelta = intval($_POST['dayDelta']);

        if(0 == $dayDelta){
            self::_error('没有任何修改');
        }

        $task = $this->loadModel('calendar')->getTask($taskID);
        if(empty($task)){
            self::_error('任务不存在');
        }

        $estStarted = self::_calDay($task->estStarted,$dayDelta);
        $deadline   = self::_calDay($task->deadline,$dayDelta);

        $result = $this->loadModel('calendar')->moveTask($taskID,$estStarted,$deadline);
        if(!$result){
            self::_error('操作失败');
        }
        self::_success();
    }

    private static function _calDay($date,$dayDelta){
        if("" == $date){
            return "";
        }
        $new_unix = strtotime($date) + (intval($dayDelta) * 86400);
        $new_date = date("Y-m-d",$new_unix);
        return $new_date;
    }

    private static function _error($info='',$data='')
    {
        $opt = array('status'=>false,'info'=>$info,'data'=>$data);
        die(json_encode($opt));
    }

    private static function _success($info='',$data='')
    {
        $opt = array('status'=>true,'info'=>$info,'data'=>$data);
        die(json_encode($opt));
    }
}
